﻿;/////////////////////////////////////////////////////////////////////////////////
;Demo of the OOP class for implementing various kinds of dynamic arrays.

;By Stephen Rodriguez.

;Demonstrates some of the methods exposed by the 'StructuredArrayObject' class.
;/////////////////////////////////////////////////////////////////////////////////

XIncludeFile "arrayClass.pbi"


;Declare our test structure.
  Structure _testStructure
    str1.s
    str2.s
    a.l
    b.w
  EndStructure

;Define and initialise 2 variables of this structured type.
;NOTE THAT (as explained in the user guide) string fields must be placed before all
;other fields.
  TEST._testStructure
    TEST\str1 = "Dylan"
    TEST\str2 = "rules!"
    TEST\a = 100
    TEST\b = 40
  newTest._testStructure
    newTest\str1 = "I am a"
    newTest\str2 = "...rocking rolling stone!"
    newTest\a = 1000
    newTest\b = 10

;Create a structured array object with room for 11 elements.
  MyArrayObject.StructuredArrayObject
  MyArrayObject = NewStructuredArray(10, SizeOf(_testStructure), 2)  ;2 strings fields in the structure.

If MyArrayObject ;Always check that the object was created successfully.

  Debug "The array object has an upper bound of : " + Str(MyArrayObject\GetUpperBound())

;Write our two structures to the array.
  MyArrayObject\SetValue(0, TEST)
  MyArrayObject\SetValue(1, newTest)

;Let us switch them over.
  MyArrayObject\GetValue(1, TEST)
  MyArrayObject\GetValue(0, newTest)

;Check the fields of TEST.
  Debug "TEST\str1 -> " + TEST\str1
  Debug "TEST\str2 -> " + TEST\str2
  Debug "TEST\a -> " + Str(TEST\a)
  Debug "TEST\b -> " + Str(TEST\b)


  MyArrayObject\ShiftElementsRight(0)
;The newly vacated zero-th element should contain a 'zeroed' structure.
  MyArrayObject\GetValue(0, TEST)

;Check the fields of TEST to check that it is indeed a 'zeroed' structure.
  Debug "TEST\str1 -> " + TEST\str1
  Debug "TEST\str2 -> " + TEST\str2
  Debug "TEST\a -> " + Str(TEST\a)
  Debug "TEST\b -> " + Str(TEST\b)

;Let us redimension to 101 elements.
  MyArrayObject\ReDim(100)

  Debug "The array object has an upper bound of : " + Str(MyArrayObject\GetUpperBound())

;Free the structured array object.
  MyArrayObject\Destroy()

Else
  MessageRequester("Error :", "There was a memory allocation problem creating the stack object.")
EndIf
; IDE Options = PureBasic 4.40 (Windows - x86)
; CursorPosition = 33
; FirstLine = 26